function demo5h
% E.W. Gekeler, Release 08.08.10 
% Example with exact solution after Boukir
% Taylor-Hood elements with convection term
% Fractional-step method, Turek, p. 165
% COMPLETE NEWTON METHOD, ALSO FOR P, LUMPED MASS MATRIX
% X,Y : coordinates
% U,V :  velocity in x- and y-direction
% P   :  Pressure
% p,e,t: nodes,edges,triangles
% p1,t1: data of midpoints and their indices in triangles
% e is augmented by an eighth row with indices of midpoints in [p,p1]
% FF1 : File for first mesh (not used in MATLAB TOOLBOX)
% FF2 : File for geometry in MATLAB format 
% FF3 : File for boundary conditions and loads  
% OPTION_MESH = 1/2 Mesh without/with MATLAB TOOLBOX
% specify one node value for pressure, cf. bsp05h.m 

clear, clc, format short, format compact
% Example:
FF1 = 'bsp05'; FF2 = 'bsp05g'; FF3 = 'bsp05h';
% Segment nrs. for outer boundary = [1,2,3,4]
% -- Parameters -------------------
b = 10;    % after Boukir
nu = 1/b;  % coeff. of viscosity [m*m/sec]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
maxiter = 20;   % step number for time iteration, 
DT      = 0.05  ;       % time step  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
OPTION_MESH = 1; REFINE = 2; % number of uniform mesh refinements
%OPTION_MESH = 2; REFINE = 3; 
theta1 = 1 - sqrt(2)/2; theta2 = 1 - 2*theta1; alfa = (1 - 2*theta1)/(1 - theta1);
beta   = 1 - alfa; theta3 = alfa*theta1*DT;
% ---------------------------------------------
Start = 100; 
while ~ismember(Start,[0,1])
   Start = input(' New start or Restart ? (1/0) ');
end
%Start = 1;
if Start == 1, 
   [p,e,t] = start4stokes(FF1,FF2,OPTION_MESH,REFINE); 
   [p1,e,t1]  = mesh06_t(p,e,t); % e is enlarged by a further row 8
   [S,C,D,MM] = taylor_hood(p,t,p1,t1);
   N1 = size(p,2); N = N1 + size(p1,2); NULL = zeros(N,N);
   % -- Dirichlet boundary conditions --------------------
   [RDU,RDV,RDP,FU,FV] = feval(FF3,p,e,p1,b);
   RD = [RDU(1,:),RDV(1,:) + N, 2*N + RDP(1);
         RDU(2,:),RDV(2,:), RDP(2)];
   RD1 = RD(1,:);      
   FF = [MM*FU;MM*FV]; 
   % -- lumped mass matrix -------------
   MM1 = diag(MM); MM = spdiags(MM1,0,N,N); 
   AA1 = [MM + (theta3*nu)*S, NULL,         - (theta1*DT)*C; 
          NULL,         MM + (theta3*nu)*S, - (theta1*DT)*D;
          C.' ,         D.',            zeros(N1,N1)];
   AA1 = sparse(AA1);   
   AA2 = [MM + (theta3*nu)*S, NULL,         - (theta2*DT)*C; 
          NULL,         MM + (theta3*nu)*S, - (theta2*DT)*D;
          C.' ,         D.',            zeros(N1,N1)];
   AA2 = sparse(AA2);   
   UU0 = zeros(2*N+N1,1); UU0(RD1) = RD(2,:).'; 

   save daten5ha p e t p1 t1  
   save daten5hb UU0  
   save daten5hc AA1 AA2 S MM C D RD FF b RD1  
end  
load daten5ha p e t p1 t1 
load daten5hb UU0  
load daten5hc AA1 AA2 S MM C D RD FF b RD1  
N1  = size(p,2); N = N1 + size(p1,2); N2 = 2*N+N1;
AUX = zeros(N2,1); AUX(RD1) = 1;
UU1 = UU0; UUA = UU0;
nsteps = 4; % number of Newton steps
tic
for I = 1:maxiter 
   % -- 1 -------------------------------------
   RSIDE1 = [(MM - (beta*theta1*DT*nu)*S)*UU0(1:N)     + (theta1*DT)*FF(1:N);
             (MM - (beta*theta1*DT*nu)*S)*UU0(N+1:2*N) + (theta1*DT)*FF(N+1:2*N)];
   CC     = triform1(p,p1,t,t1,UU0(1:N),UU0(N+1:2*N));
   RSIDE1 = RSIDE1 - (beta*theta1*DT)*CC*UU0(1:2*N);  
   RSIDE1 = [RSIDE1;zeros(N1,1)];
   for JJ = 1:nsteps  
      [CC,DD] = triform6(p,p1,t,t1,UU1,RD1);
      BB = AA1 + theta3*CC;
      CC = BB  + theta3*DD; 
      RSIDE = BB*UU1 - RSIDE1; RSIDE(RD1) = 0;
      CC(RD1,:) = 0; CC(:,RD1) = 0;
      CC  = spdiags(diag(CC)+AUX,0,CC);
      DUU = CC\RSIDE; 
      UU1 = UU1 - DUU; 
   end
   % --  2 --------------------------------
   UU0    = UU1;
   RSIDE1 = [(MM - (alfa*theta2*DT*nu)*S)*UU0(1:N)     + (theta2*DT)*FF(1:N);
             (MM - (alfa*theta2*DT*nu)*S)*UU0(N+1:2*N) + (theta2*DT)*FF(N+1:2*N)];
   CC     = triform1(p,p1,t,t1,UU0(1:N),UU0(N+1:2*N));
   RSIDE1 = RSIDE1 - (alfa*theta2*DT)*CC*UU0(1:2*N);  
   RSIDE1 = [RSIDE1;zeros(N1,1)];
   for JJ = 1:nsteps  
      [CC,DD] = triform6(p,p1,t,t1,UU1,RD1);
      BB = AA2 + theta3*CC;
      CC = BB  + theta3*DD; 
      RSIDE = BB*UU1 - RSIDE1; RSIDE(RD1) = 0;
      CC(RD1,:) = 0; CC(:,RD1) = 0;
      CC  = spdiags(diag(CC)+AUX,0,CC);
      DUU = CC\RSIDE; 
      UU1 = UU1 - DUU; 
   end
   % --  3 ---------------------------------
   UU0    = UU1;
   RSIDE1 = [(MM - (beta*theta1*DT*nu)*S)*UU0(1:N)     + (theta1*DT)*FF(1:N);
             (MM - (beta*theta1*DT*nu)*S)*UU0(N+1:2*N) + (theta1*DT)*FF(N+1:2*N)];
   CC     = triform1(p,p1,t,t1,UU0(1:N),UU0(N+1:2*N));
   RSIDE1 = RSIDE1 - (beta*theta1*DT)*CC*UU0(1:2*N);  
   RSIDE1 = [RSIDE1;zeros(N1,1)];
   for JJ = 1:nsteps  
      [CC,DD] = triform6(p,p1,t,t1,UU1,RD1);
      BB = AA1 + theta3*CC;
      CC = BB  + theta3*DD; 
      RSIDE = BB*UU1 - RSIDE1; RSIDE(RD1) = 0;
      CC(RD1,:) = 0; CC(:,RD1) = 0;
      CC  = spdiags(diag(CC)+AUX,0,CC);
      DUU = CC\RSIDE; 
      UU1 = UU1 - DUU; 
   end

   %DIFF_N = max(abs(DUU))
   %RESMAX = max(abs(RSIDE))
   DIFF = max(abs(UU1 - UUA));  I_DIFF = [I,DIFF]
   UU0 = UU1; UUA = UU0; 
end
toc
save daten5hb UU0  
disp(' Call bild05 and select demo5h ! ')
clear all