function [SE,ME,CE,DE,BE] = thlsto_t(X,Y)
% Supplies element matrices for quadratic elements
% SE(6,6), ME(6,6), CE(6,6), DE(6,6), BE(6,1)
% for a TAYLOR-HOOD element in TRANSIENT Stokes problems
% U and P both quadratic
% INPUT:
% X,Y Coordinates of the three vertices
% ecode = 1 for wrong orientation

ecode = 0; SE = []; ME = []; CE = []; DE = []; BE = [];

S1 = [3   1   0  -4   0   0;
      1   3   0  -4   0   0;
      0   0   0   0   0   0;
     -4  -4   0   8   0   0;
      0   0   0   0   8  -8;
      0   0   0   0  -8   8];
S2 = [6   1   1  -4   0  -4;
      1   0  -1  -4   4   0;
      1  -1   0   0   4  -4;
     -4  -4   0   8  -8   8;
      0   4   4  -8   8  -8;
     -4   0  -4   8  -8   8];
S3 = [3   0   1   0   0  -4;
      0   0   0   0   0   0;
      1   0   3   0   0  -4;
      0   0   0   8  -8   0;
      0   0   0  -8   8   0;
     -4   0  -4   0   0   8];

S4 = [6  -1  -1   0  -4   0;
     -1   6  -1   0   0  -4;
     -1  -1   6  -4   0   0;
      0   0  -4  32  16  16;
     -4   0   0  16  32  16;
      0  -4   0  16  16  32];
SB = [0;  0;  0;  1;  1;  1];
      
  
C1 = ...
 -2,  1,  1, -3,  1, -3;
 -1,  2, -1,  3,  3, -1;
  0,  0,  0,  0,  0,  0;
  3, -3,  0,  0, -4,  4;
 -1, -1,  2,  4,  8,  8;
  1,  1, -2, -4, -8, -8]/30; %for DP/DX
C2 = ...
 -2,  1,  1, -3,  1, -3;
  0,  0,  0,  0,  0,  0;
 -1, -1,  2, -1,  3,  3;
  1, -2,  1, -8, -8, -4;
 -1,  2, -1,  8,  8,  4;
  3,  0, -3,  4, -4,  0]/30; % for DP/DY
  
ecode = 0;
X21 = X(2) - X(1); X32 = X(3) - X(2); X31 = X(3) - X(1); X13 = - X31;
Y21 = Y(2) - Y(1); Y32 = X(3) - X(2); Y31 = Y(3) - Y(1); Y12 = - Y21;
XX = [X21, X32, X31]; YY = [Y21, Y32, Y31];
DET = X21*Y31 - X31*Y21;
if DET > 0
   A  =  (X31*X31 + Y31*Y31)/DET;
   B  = -(X31*X21 + Y31*Y21)/DET;
   C  =  (X21*X21 + Y21*Y21)/DET;
   SE =  (A*S1 + B*S2 + C*S3)/6; % stiffness matrix
   ME =  DET*S4/360;             % Mass matrix
   CE =  (Y31*C1 + Y12*C2)/6; 
   DE =  (X13*C1 + X21*C2)/6;
   BE =  DET*SB/6;   % load vector for quadratic elements
else ecode = 1; return    
end
